from .add_emails_to_addressbook import AddEmailsToAddressbook
from .create_addressbook import CreateAddressbook
from .edit_addressbook import EditAddressbook
from .get_addressbook import GetAddressbook
from .get_emails_from_addressbook import GetEmailsFromAddressbook
from .get_list_of_addressbook import GetListOfAddressbook
from .get_list_of_addressbook_variables import GetListOfAddressbookVariables
from .get_total_emails import GetTotalEmails
from .search_emails import SearchEmails


__all__ = [
    "AddEmailsToAddressbook",
    "GetListOfAddressbook",
    "GetEmailsFromAddressbook",
    "CreateAddressbook",
    "EditAddressbook",
    "GetAddressbook",
    "GetListOfAddressbookVariables",
    "GetTotalEmails",
    "SearchEmails",
]



from typing import Union

from aiosendpulse.methods.email.templates import CreateTemplate, EditTemplate, GetTemplate
from aiosendpulse.services.base import BaseService
from aiosendpulse.types import CreateTemplateResult, Result, Template


__all__ = ["TemplateService"]


class TemplateService(BaseService):
    async def create(self, html: str, lang: str, name: str = None) -> CreateTemplateResult:
        return await CreateTemplate(
            name=name,
            body=html,
            lang=lang,  # noqa
        )(client=self.http_client, auth=self.auth)

    async def edit(self, template_id: Union[str, int], html: str) -> Result:
        return await EditTemplate(
            id=template_id,
            body=html,
        )(client=self.http_client, auth=self.auth)

    async def get(self, template_id: Union[str, int]) -> Template:
        return await GetTemplate(
            template_id=template_id,
        )(client=self.http_client, auth=self.auth)



from typing import Union

from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import Addressbook


__all__ = ["GetListOfAddressbook"]


class GetListOfAddressbook(SendPulseMethod[list[Addressbook]]):
    limit: Union[int, None] = None
    offset: Union[int, None] = None

    __http_method__ = "GET"
    __api_endpoint__ = "/addressbooks"
    __returning__ = list[Addressbook]

    def build_request(self, base_url: URL) -> Request:
        params = {}
        if self.limit is not None:
            params["limit"] = self.limit

        if self.offset is not None:
            params["offset"] = self.offset

        return Request(
            method=self.__http_method__,
            url=base_url.join(self.__api_endpoint__),
            params=params,
        )



from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import Addressbook


__all__ = ["GetAddressbook"]


class GetAddressbook(SendPulseMethod[list[Addressbook]]):
    id: int

    __http_method__ = "GET"
    __api_endpoint__ = "/addressbooks/{id}"
    __returning__ = list[Addressbook]

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__.format(id=self.id)),
        )



from __future__ import annotations

from typing import ClassVar, Generic, Literal, TypeVar, Union, get_args, get_origin

from httpx import URL, AsyncClient, Auth, HTTPError, HTTPStatusError, Request

from aiosendpulse.exceptions import ExceptionDispatcher
from aiosendpulse.logger import logger
from aiosendpulse.types.base import SendPulseObject


__all__ = ["SendPulseMethod"]


SendPulseType = TypeVar("SendPulseType")


class SendPulseMethod(SendPulseObject, Generic[SendPulseType]):
    __http_method__: ClassVar[str]
    __api_endpoint__: ClassVar[str]
    __returning__: ClassVar[type]
    __query_params__: ClassVar[set[str]] = set()
    __path_params__: ClassVar[set[str]] = set()
    __body_params__: ClassVar[set[str]] = set()
    __content_type__: ClassVar[Literal["application/json"]] = "application/json"

    def build_request(self, base_url: URL) -> Request:
        raise NotImplementedError()

    async def __call__(self, client: AsyncClient, auth: Auth = None) -> Union[SendPulseType, dict, None]:
        request = self.build_request(base_url=client.base_url)
        try:
            response = await client.send(request, auth=auth)
        except HTTPError as e:
            logger.error(f"Calling method {self.__api_endpoint__} falling with exception: {e}")
        else:
            try:
                response.raise_for_status()
            except HTTPStatusError:
                data = response.json()
                exception = ExceptionDispatcher.get(error_code=data.get("error_code"), **self.model_dump())
                raise exception

            if get_origin(self.__returning__) is list:
                args = get_args(self.__returning__)

                if len(args) != 1:
                    raise ValueError("generic type must be with one inner annotation class")

                args = args[0]

                if issubclass(args, SendPulseObject):
                    return [args.model_validate(obj=obj) for obj in response.json()]

            if issubclass(self.__returning__, SendPulseObject):
                return self.__returning__.model_validate_json(json_data=response.content)

            return response.json()



from typing import Literal

from .base import MutableSendPulseObjectObject


__all__ = ["AddressbookVariable"]


class AddressbookVariable(MutableSendPulseObjectObject):
    name: str
    type: Literal["string", "date", "number"]



from abc import ABC, abstractmethod
from datetime import timedelta
from typing import Union


__all__ = ["BaseTokenStorage"]


class BaseTokenStorage(ABC):
    @abstractmethod
    async def set(self, token: str, ex: Union[int, timedelta]) -> None:
        raise NotImplementedError()

    @abstractmethod
    async def get(self) -> Union[str, None]:
        raise NotImplementedError()



from typing import Union

from aiosendpulse.methods.smtp import SendEmail
from aiosendpulse.services.base import BaseService
from aiosendpulse.types import Recipient, Result, Sender, TemplateMeta


__all__ = ["SmtpService"]


class SmtpService(BaseService):
    async def send_email(
        self,
        subject: str,
        sender: Union[Sender, dict],
        to: list[Union[Recipient, dict]],
        html: Union[str, None] = None,
        text: Union[str, None] = None,
        template: Union[TemplateMeta, dict] = None,
        auto_plain_text: bool = False,
        cc: Union[list[Union[Recipient, dict]], None] = None,
        bcc: Union[list[Union[Recipient, dict]], None] = None,
        attachments: Union[dict[str, str], None] = None,
        attachments_binary: Union[dict[str, str], None] = None,
    ) -> Result:
        return await SendEmail(
            html=html,
            text=text,
            template=template,
            auto_plaint_text=auto_plain_text,
            subject=subject,
            sender=sender,
            to=to,
            cc=cc,
            bcc=bcc,
            attachments=attachments,
            attachments_binary=attachments_binary,
        )(client=self.http_client, auth=self.auth)



from base64 import b64encode
from functools import cached_property
from typing import Union

from httpx import URL, Request
from pydantic import computed_field
from pydantic_extra_types.language_code import LanguageAlpha2

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import CreateTemplateResult


__all__ = ["CreateTemplate"]


class CreateTemplate(SendPulseMethod[CreateTemplateResult]):
    name: Union[str, None] = None
    body: str
    lang: LanguageAlpha2

    __http_method__ = "POST"
    __api_endpoint__ = "/template"
    __returning__ = CreateTemplateResult

    @computed_field(return_type=str)
    @cached_property
    def encoded_body(self) -> str:
        return b64encode(self.body.encode()).decode()

    def build_request(self, base_url: URL) -> Request:
        data = {
            "body": self.encoded_body,
            "lang": self.lang,
        }
        if self.name:
            data["name"] = self.name

        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__),
            json=data,
        )



from typing import Union

from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import EmailDetail


__all__ = ["GetEmailsFromAddressbook"]


class GetEmailsFromAddressbook(SendPulseMethod[list[EmailDetail]]):
    id: int
    limit: Union[int, None] = None
    offset: Union[int, None] = None
    active: Union[bool, None] = None
    not_active: Union[bool, None] = None

    __http_method__ = "GET"
    __api_endpoint__ = "/addressbooks/{id}/emails"
    __returning__ = list[EmailDetail]

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__.format(id=self.id)),
            params=self.model_dump(exclude_none=True, exclude_unset=True, exclude={"id"}),
        )



from typing import Union

from aiosendpulse.methods.email.addressbook import (
    AddEmailsToAddressbook,
    CreateAddressbook,
    EditAddressbook,
    GetAddressbook,
    GetEmailsFromAddressbook,
    GetListOfAddressbook,
    GetListOfAddressbookVariables,
    GetTotalEmails,
    SearchEmails,
)
from aiosendpulse.services.base import BaseService
from aiosendpulse.types import Addressbook, AddressbookId, AddressbookVariable, EmailDetail, Result, TotalEmails


__all__ = ["AddressbookService"]


class AddressbookService(BaseService):
    async def create(self, name: str) -> AddressbookId:
        return await CreateAddressbook(
            book_name=name,
        )(client=self.http_client, auth=self.auth)

    async def edit(self, addressbook_id: int, name: str) -> Result:
        return await EditAddressbook(id=addressbook_id, name=name)(client=self.http_client, auth=self.auth)

    async def get_list(self, limit: int = None, offset: int = None) -> list[Addressbook]:
        return await GetListOfAddressbook(
            limit=limit,
            offset=offset,
        )(client=self.http_client, auth=self.auth)

    async def get(self, addressbook_id: int) -> Addressbook:
        method = GetAddressbook(id=addressbook_id)
        response = await method(client=self.http_client, auth=self.auth)
        if response:
            return response[0]

    async def variables(self, addressbook_id: int) -> list[AddressbookVariable]:
        return await GetListOfAddressbookVariables(
            id=addressbook_id,
        )(client=self.http_client, auth=self.auth)

    async def add_emails(self, addressbook_id: int, emails: list[Union[EmailDetail, dict]]) -> Result:
        return await AddEmailsToAddressbook(
            addressbook_id=addressbook_id,
            emails=emails,
        )(client=self.http_client, auth=self.auth)

    async def get_emails(
        self, addressbook_id: int, limit: int = None, offset: int = None, active: bool = None, not_active: bool = None
    ) -> list[EmailDetail]:
        return await GetEmailsFromAddressbook(
            id=addressbook_id,
            limit=limit,
            offset=offset,
            active=active,
            not_active=not_active,
        )(client=self.http_client, auth=self.auth)

    async def get_total_emails(self, addressbook_id: int) -> TotalEmails:
        return await GetTotalEmails(id=addressbook_id)(client=self.http_client, auth=self.auth)

    async def search_emails(self, addressbook_id: int, variable_name: str, variable_value: str) -> list[EmailDetail]:
        return await SearchEmails(id=addressbook_id, variable_name=variable_name, search_value=variable_value)(
            client=self.http_client, auth=self.auth
        )



from datetime import timedelta
from typing import Union

from redis import ConnectionError as RedisConnectionError
from redis.asyncio import Redis

from .base import BaseTokenStorage
from .exceptions import CacheUnavailableError


__all__ = ["RedisTokenStorage"]


class RedisTokenStorage(BaseTokenStorage):
    def __init__(self, redis: Redis, key: str = "SENDPULSE_TOKEN") -> None:
        self.redis = redis
        self.key = key

    async def set(self, token: str, ex: Union[int, timedelta] = 3600) -> None:
        try:
            await self.redis.set(name=self.key, value=token, ex=ex)
        except RedisConnectionError as e:
            raise CacheUnavailableError from e

    async def get(self) -> Union[str, None]:
        try:
            token = await self.redis.get(name=self.key)
            if token:
                return token.decode()
        except RedisConnectionError as e:
            raise CacheUnavailableError from e



from pydantic import BaseModel, ConfigDict


class SendPulseObject(BaseModel):
    model_config = ConfigDict(
        use_enum_values=True,
        extra="allow",
        validate_assignment=True,
        frozen=True,
        populate_by_name=True,
        arbitrary_types_allowed=True,
        defer_build=True,
    )


class MutableSendPulseObjectObject(SendPulseObject):
    model_config = ConfigDict(
        frozen=False,
    )



from __future__ import annotations

from datetime import datetime
from typing import TYPE_CHECKING, Union

from pydantic import HttpUrl
from pydantic_extra_types.language_code import LanguageAlpha2

from .base import MutableSendPulseObjectObject


if TYPE_CHECKING:
    from .template_category import TemplateCategory


__all__ = ["Template", "TemplateMeta"]


class Template(MutableSendPulseObjectObject):
    id: str
    real_id: int
    lang: LanguageAlpha2
    name: str
    name_slug: Union[str, None] = None
    created: datetime
    full_description: Union[str, None] = None
    is_structured: bool
    category: str
    category_info: TemplateCategory
    tags: dict[str, str] = {}
    owner: str
    preview: HttpUrl


class TemplateMeta(MutableSendPulseObjectObject):
    id: Union[str, int]
    variables: dict[str, str]



from pydantic import SecretStr

from .base import MutableSendPulseObjectObject


__all__ = ["Token"]


class Token(MutableSendPulseObjectObject):
    access_token: SecretStr
    token_type: str
    expires_in: int



from __future__ import annotations

from typing import Literal, Union

from httpx import AsyncClient

from aiosendpulse.auth import BearerTokenAuth
from aiosendpulse.logger import logger
from aiosendpulse.methods import Authorize
from aiosendpulse.services.email import EmailService
from aiosendpulse.services.smtp import SmtpService
from aiosendpulse.storage.base import BaseTokenStorage
from aiosendpulse.storage.exceptions import CacheUnavailableError


__all__ = ["AioSendPulseClient"]


BASE_URL: str = "https://api.sendpulse.com"


class AioSendPulseClient:
    def __init__(
        self,
        client_id: str,
        client_secret: str,
        token_storage: BaseTokenStorage,
        grant_type: Literal["client_credentials"] = "client_credentials",
        token: str = None,
    ) -> None:
        self.http_client = AsyncClient(base_url=BASE_URL)
        self.auth_class: type[BearerTokenAuth] = BearerTokenAuth
        self.__token_cache_backend = token_storage
        self.__client_id = client_id
        self.__client_secret = client_secret
        self.__grant_type = grant_type
        self.__auth: Union[BearerTokenAuth, None] = None

        if token is not None:
            self.__auth = self.auth_class(token=token)

    @property
    def token(self) -> Union[str, None]:
        if self.__auth is not None:
            return self.__auth.token

    @property
    def auth(self) -> Union[BearerTokenAuth, None]:
        return self.__auth

    async def close(self) -> None:
        await self.http_client.aclose()

    async def authorize(self) -> None:
        try:
            token = await self.__token_cache_backend.get()
        except CacheUnavailableError as e:
            logger.error(f"Token cache unavailable with error: {e}")
            token = None

        if token is None:
            logger.debug("SendPulse token not cached")
            method = Authorize(
                client_id=self.__client_id,
                client_secret=self.__client_secret,
                grant_type=self.__grant_type,
            )
            response = await method(client=self.http_client)
            token = response.access_token.get_secret_value()
            ex = response.expires_in

            try:
                await self.__token_cache_backend.set(token=token, ex=ex)
            except CacheUnavailableError as e:
                logger.error(f"Token cache unavailable with error: {e}")
            else:
                logger.debug("Cache SendPulse token")

        self.__auth = self.auth_class(
            token=token,
        )

    @property
    def email(self) -> EmailService:
        return EmailService(client=self)

    @property
    def smtp(self) -> SmtpService:
        return SmtpService(client=self)



from aiosendpulse.services.base import BaseService
from aiosendpulse.services.email.addressbook import AddressbookService
from aiosendpulse.services.email.blacklist import BlacklistService
from aiosendpulse.services.email.template import TemplateService


__all__ = ["EmailService"]


class EmailService(BaseService):
    @property
    def addressbook(self) -> AddressbookService:
        return AddressbookService(client=self.client)

    @property
    def template(self) -> TemplateService:
        return TemplateService(client=self.client)

    @property
    def blacklist(self) -> BlacklistService:
        return BlacklistService(client=self.client)



from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import Result


__all__ = ["EditAddressbook"]


class EditAddressbook(SendPulseMethod[Result]):
    id: int
    name: str

    __http_method__ = "PUT"
    __api_endpoint__ = "/addressbooks/{id}"
    __returning__ = Result

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__.format(id=self.id)),
            json={
                "name": self.name,
            },
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Union

from httpx import AsyncClient


if TYPE_CHECKING:
    from aiosendpulse import AioSendPulseClient
    from aiosendpulse.auth import BearerTokenAuth


__all__ = ["BaseService"]


class BaseService:
    def __init__(self, client: AioSendPulseClient) -> None:
        self.client = client

    @property
    def http_client(self) -> AsyncClient:
        return self.client.http_client

    @property
    def auth(self) -> Union[BearerTokenAuth, None]:
        return self.client.auth



import typing

from httpx import Auth, Request, Response


__all__ = ["BearerTokenAuth"]


class BearerTokenAuth(Auth):
    def __init__(self, token: str, header: str = "Authorization") -> None:
        self.__token = token
        self.header = header

    def auth_flow(self, request: Request) -> typing.Generator[Request, Response, None]:
        request.headers[self.header] = f"Bearer {self.__token}"
        yield request

    @property
    def token(self) -> str:
        return self.__token



from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import AddressbookId


__all__ = ["CreateAddressbook"]


class CreateAddressbook(SendPulseMethod[AddressbookId]):
    book_name: str

    __http_method__ = "POST"
    __api_endpoint__ = "/addressbooks"
    __returning__ = AddressbookId

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__, url=base_url.join(url=self.__api_endpoint__), json={"bookName": self.book_name}
        )



from typing import Literal

from httpx import URL, Request

from aiosendpulse.types import Token

from .base import SendPulseMethod


__all__ = ["Authorize"]


class Authorize(SendPulseMethod[Token]):
    grant_type: Literal["client_credentials"]
    client_id: str
    client_secret: str

    __http_method__ = "POST"
    __api_endpoint__ = "/oauth/access_token"
    __returning__ = Token

    def build_request(self, base_url: URL):
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__),
            json={
                "grant_type": self.grant_type,
                "client_id": self.client_id,
                "client_secret": self.client_secret,
            },
        )



from typing import Union

from .base import MutableSendPulseObjectObject


__all__ = ["TemplateCategory"]


class TemplateCategory(MutableSendPulseObjectObject):
    id: int
    name: str
    meta_description: Union[str, None] = None
    full_description: Union[str, None] = None
    code: str
    sort: int



from base64 import b64encode
from functools import cached_property

from httpx import URL, Request
from pydantic import EmailStr, computed_field

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import Result


class DeleteEmailsFromBlacklist(SendPulseMethod[Result]):
    emails: list[EmailStr]

    __http_method__ = "DELETE"
    __api_endpoint__ = "/blacklist"
    __returning__ = Result

    @computed_field(return_type=str)
    @cached_property
    def encoded_emails(self) -> str:
        return b64encode(",".join(self.emails).encode()).decode()

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__),
            json={
                "emails": self.encoded_emails,
            },
        )



from typing import Union

from pydantic import EmailStr

from .base import SendPulseObject


__all__ = ["EmailDetail"]


class EmailDetail(SendPulseObject):
    email: EmailStr
    status: Union[int, None] = None
    status_explain: Union[str, None] = None
    variables: Union[dict[str, str], None] = None



from .add_emails_to_blacklist import AddEmailsToBlacklist
from .delete_emails_from_blacklist import DeleteEmailsFromBlacklist


__all__ = [
    "AddEmailsToBlacklist",
    "DeleteEmailsFromBlacklist",
]



from __future__ import annotations


class ExceptionDispatcher:
    exceptions: dict[int, SendPulseException] = {}

    def __init_subclass__(cls, **kwargs):
        cls.exceptions[cls.error_code] = cls  # noqa

    @classmethod
    def get(cls, error_code: int, **kwargs) -> SendPulseException:
        exc_class = cls.exceptions.get(error_code, SendPulseException)
        exc = exc_class()
        exc.message = exc.message.format_map(kwargs)
        return exc


class SendPulseException(Exception):
    error_code: int

    def __init__(self) -> None:
        self.message: str = "Something went wrong"

    def __str__(self) -> str:
        return f"{self.message}"


class NoDataError(ExceptionDispatcher, SendPulseException):
    error_code = 8

    def __init__(self) -> None:
        self.message = "No data"


class SenderEmailAddressMissingError(ExceptionDispatcher, SendPulseException):
    error_code = 10

    def __init__(self) -> None:
        self.message = "Sender email address is missing"


class RecipientsAddressesNotFoundError(ExceptionDispatcher, SendPulseException):
    error_code = 11

    def __init__(self) -> None:
        self.message = "Recipients addresses not found"


class EmailNotFoundError(ExceptionDispatcher, SendPulseException):
    error_code = 17

    def __init__(self) -> None:
        self.message = "Email not found"


class AddressbookNotFoundError(ExceptionDispatcher, SendPulseException):
    error_code = 213

    def __init__(self) -> None:
        self.message = "Addressbook {addressbook_id} not found"



from datetime import datetime

from .base import MutableSendPulseObjectObject


__all__ = ["Addressbook", "AddressbookId"]


class AddressbookId(MutableSendPulseObjectObject):
    id: int


class Addressbook(AddressbookId):
    name: str
    all_email_qty: int
    active_email_qty: int
    inactive_email_qty: int
    active_phones_quantity: int
    exc_phones_quantity: int
    creationdate: datetime
    status: int
    status_explain: str



from base64 import b64encode
from functools import cached_property
from typing import Union

from httpx import URL, Request
from pydantic import computed_field

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import Result


__all__ = ["EditTemplate"]


class EditTemplate(SendPulseMethod[Result]):
    id: Union[str, int]
    body: str

    __http_method__ = "POST"
    __api_endpoint__ = "/template/edit/{id}"
    __returning__ = Result

    @computed_field(return_type=str)
    @cached_property
    def encoded_body(self) -> str:
        return b64encode(self.body.encode()).decode()

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__.format(id=self.id)),
            json={
                "body": self.encoded_body,
            },
        )



from base64 import b64encode
from functools import cached_property
from typing import Union

from httpx import URL, Request
from pydantic import EmailStr, computed_field

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import Result


class AddEmailsToBlacklist(SendPulseMethod[Result]):
    emails: list[EmailStr]
    comment: Union[str, None] = None

    __http_method__ = "POST"
    __api_endpoint__ = "/blacklist"
    __returning__ = Result

    @computed_field(return_type=str)
    @cached_property
    def encoded_emails(self) -> str:
        return b64encode(",".join(self.emails).encode()).decode()

    def build_request(self, base_url: URL) -> Request:
        data = {
            "emails": self.encoded_emails,
        }

        if self.comment:
            data["comment"] = self.comment

        return Request(method=self.__http_method__, url=base_url.join(url=self.__api_endpoint__), json=data)



from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import AddressbookVariable


__all__ = ["GetListOfAddressbookVariables"]


class GetListOfAddressbookVariables(SendPulseMethod[list[AddressbookVariable]]):
    id: int

    __http_method__ = "GET"
    __api_endpoint__ = "/addressbooks/{id}/variables"
    __returning__ = list[AddressbookVariable]

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__.format(id=self.id)),
        )



from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import EmailDetail, Result


__all__ = ["AddEmailsToAddressbook"]


class AddEmailsToAddressbook(SendPulseMethod[Result]):
    addressbook_id: int
    emails: list[EmailDetail]

    __http_method__ = "POST"
    __api_endpoint__ = "/addressbooks/{addressbook_id}/emails"
    __returning__ = dict

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__.format(addressbook_id=self.addressbook_id)),
            json=self.model_dump(mode="json", exclude_none=True, exclude_unset=True),
        )



from aiosendpulse.methods.email.blacklist import AddEmailsToBlacklist, DeleteEmailsFromBlacklist
from aiosendpulse.services.base import BaseService
from aiosendpulse.types import Result


__all__ = ["BlacklistService"]


class BlacklistService(BaseService):
    async def add_emails_to_blacklist(self, emails: list[str], comment: str = None) -> Result:
        return await AddEmailsToBlacklist(
            emails=emails,
            comment=comment,
        )(client=self.http_client, auth=self.auth)

    async def delete_emails_from_blacklist(self, emails: list[str]) -> Result:
        return await DeleteEmailsFromBlacklist(emails=emails)(client=self.http_client, auth=self.auth)



from __future__ import annotations

from base64 import b64encode
from typing import Annotated, Union

from httpx import URL, Request
from pydantic import Field, ValidationError, field_validator, model_validator

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import Recipient, Result, Sender, TemplateMeta


__all__ = ["SendEmail"]


class SendEmail(SendPulseMethod[Result]):
    html: Union[str, None] = None
    text: Union[str, None] = None
    template: Union[TemplateMeta, None] = None
    auto_plaint_text: bool = False
    subject: str
    sender: Annotated[Sender, Field(serialization_alias="from")]
    to: list[Recipient]
    cc: Union[list[Recipient], None] = None
    bcc: Union[list[Recipient], None] = None
    attachments: Union[dict[str, str], None] = None
    attachments_binary: Union[dict[str, str], None] = None

    __http_method__ = "POST"
    __api_endpoint__ = "/smtp/emails"
    __returning__ = Result

    @model_validator(mode="after")
    def model_validator(self) -> SendEmail:
        if self.html and self.template:
            raise ValidationError("")

        if not self.html and not self.template:
            raise ValidationError("")

        return self

    @field_validator("attachments_binary", mode="after")  # noqa
    @classmethod
    def attachment_binary_validator(cls, v: Union[dict[str, str], None]) -> Union[dict[str, str], None]:
        if v is None:
            return None

        return {k: b64encode(v.encode()).decode() for k, v in v.items()}

    @field_validator("html", mode="after")  # noqa
    @classmethod
    def html_validator(cls, v: Union[str, None]) -> Union[str, None]:
        if v is None:
            return None

        return b64encode(v.encode()).decode()

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__),
            json={
                "email": self.model_dump(mode="json", exclude_none=True, exclude_unset=True, by_alias=True),
            },
        )



from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import TotalEmails


__all__ = ["GetTotalEmails"]


class GetTotalEmails(SendPulseMethod[TotalEmails]):
    id: int

    __http_method__ = "GET"
    __api_endpoint__ = "/addressbooks/{id}/emails/total"

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__),
        )



from typing import Union

from httpx import URL, Request

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import Template


__all__ = ["GetTemplate"]


class GetTemplate(SendPulseMethod[Template]):
    template_id: Union[int, str]

    __http_method__ = "GET"
    __api_endpoint__ = "/template/{template_id}"
    __returning__ = Template

    def build_request(self, base_url: URL) -> Request:
        return Request(
            method=self.__http_method__,
            url=base_url.join(url=self.__api_endpoint__.format(template_id=self.template_id)),
        )



from .addressbook import Addressbook, AddressbookId
from .addressbook_variable import AddressbookVariable
from .email import EmailDetail
from .recipient import Recipient
from .result import CreateTemplateResult, Result
from .sender import Sender
from .template import Template, TemplateMeta
from .template_category import TemplateCategory
from .token import Token
from .total_emails import TotalEmails


Template.model_rebuild()


__all__ = [
    "Addressbook",
    "AddressbookId",
    "AddressbookVariable",
    "EmailDetail",
    "Recipient",
    "CreateTemplateResult",
    "Result",
    "Sender",
    "Template",
    "TemplateMeta",
    "TemplateCategory",
    "Token",
    "TotalEmails",
]



from typing import Union

from .base import MutableSendPulseObjectObject


__all__ = ["Result", "CreateTemplateResult"]


class Result(MutableSendPulseObjectObject):
    result: bool
    id: Union[str, int, None] = None


class CreateTemplateResult(Result):
    real_id: int



from typing import Annotated

from httpx import URL, Request
from pydantic import Field

from aiosendpulse.methods.base import SendPulseMethod
from aiosendpulse.types import EmailDetail


__all__ = ["SearchEmails"]


class SearchEmails(SendPulseMethod[list[EmailDetail]]):
    id: int
    variable_name: Annotated[str, Field(serialization_alias="variableName")]
    search_value: Annotated[str, Field(serialization_alias="searchValue")]

    __http_method__ = "GET"
    __api_endpoint__ = "/addressbooks/{id}/variables/{variable_name}/emails/{search_value}"
    __returning__ = list[EmailDetail]
    __path_params__ = {"id", "variable_name", "search_value"}

    def build_request(self, base_url: URL) -> Request:
        data = {}
        if self.__body_params__:
            data = {
                "json" if self.__content_type__ == "application/json" else "body": self.model_dump(
                    include=self.__body_params__,
                    exclude_none=True,
                    exclude_unset=True,
                    by_alias=True,
                ),
            }
        return Request(
            method=self.__http_method__,
            url=base_url.join(
                url=self.__api_endpoint__.format_map(
                    self.model_dump(include=self.__path_params__) if self.__path_params__ else {}
                ),
            ),
            params=self.model_dump(include=self.__query_parameters__) if self.__query_parameters__ else {},
            headers={"content-type": self.__content_type__},
            **data,
        )
